/*---------------------------------------------------------------------------------------
|
|   [Thermometer mit Logger und grafischem Display]
| 	Copyright (C) [2022]  	[Clemens Verstappen DL3ETW] 
| 
|	Controller ATmega328P mit OLED Display SH1106 128*64 px
| 	Temperature sensor SIG306, Encoder 
|	
|	Fuse: 8 MHz internel RC oszillator, 
|
|	SEE Calculation for TSic 306 and TSic 506 ! DIFFERENCE both types are supported, 
|	modify code at three position below. TSic 306 is now selected (accuracy 0.3 kelvin)
|
|   Function: 
|	In MEASURE mode display of present temperatur from -50 to +150 C 
| 	In GRAPH mode display of 100 logged temperatures from an float array 
|   Logging starts automatically. Display is switched off and controller go into sleep 
|	power safe mode. Each 15 minutes the present temperatur is measured 
|	an stored in an array of 100 datapoints.
|	Controller button resumes device in to operating GRAPH mode
|	and displays recent data collected.
|	Current consuption: 6.5 uA (!) in sleep mode.
|
|	Version:
|	2.0		Spannungsversorgung LCD ber Controller PB2, 
|	
|---------------------------------------------------------------------------------------- */

#include  	"my_defines_mega328.h"
#include 	<avr/io.h>
#include 	<stdio.h>
#include 	<util/delay.h>
#include 	<avr/interrupt.h>
#include 	<string.h> 				// 
#include 	<stdlib.h>
#include 	<stdbool.h>				// boolsche algebra
#include 	<avr/sleep.h>			// for sleep modes
#include 	<avr/wdt.h>				// for watchdog
#include 	<avr/power.h>			// and power down mode
#include 	"my_uart.h"				// serial for debugging

#include 	"oled_lcd.h"
#include 	"oled_font.h"
#include 	"my_i2c.h"

//#define		DEBUG				// UART and LED on, while debugging

static volatile bool timer_flag = false,			// main intervall e.g. 200 ms or 10 ms
 					button_flag = false,			// button of encoder switch pressed
					encoder_left_flag = false,		// encoder left turn interrupt
					encoder_right_flag = false,
					A_set = true,					// level on encoder A
					B_set = true,					// level on encoder B
					watchdog_flag = false,
					power_down_mode = false;

static 	volatile	uint8_t 	marker = 20, marker_old = 20, watchdog_counter = 0,
								charMode = MEASURE, charMode_old = GRAPH; 	// memorise change of mode

static 	volatile 	uint16_t	voltage_pixel_old = 0, delta_px = 0, voltage_pixel = 0,
								auto_sleep_counter = 0;

					float		history[101];		// array temperatur
					float		temperature_ft;		// working variable temperatur
					char 		display_str[21];	// array for strings
  					char 		temp_str[21];

void watchdogSetup(void);
uint8_t calculate_min(void);
uint8_t calculate_max(void);		
void draw_graph(void);					 
void welcome_lcd(void);					// welcome symbole quad
void draw_coordinate_system_lcd(void);	// draw 
uint16_t getTSicTemp(void);				// read sensor
void display_temp_OLED(void);			// 

// ******************************  MAIN  ***********************************

int main(void) 	
{
// 	init port pins direction all output except ENC_A, ENC_B, ENC_SW, RESET, SENSOR_SIGNAL
//					bit 7				6				5				  4				   3			    2				    1			        	0
	DDRB 	|= (1<<UNUSED_PB7) |(1<<UNUSED_PB6) |(1<<UNUSED_PB5)  |(1<<UNUSED_PB4)   |(1<<UNUSED_PB3)    |(1<<LCD_POWER)					     |(1<<LEBENSLICHT);
	DDRC	|= 								     (1<<I2C_SCL)     |(1<<I2C_SDA)      |                    (1<<SENSOR_POWER)     |(1<<UNUSED_PC1) |(1<< UNUSED_PC0);
	DDRD	|= (1<<UNUSED_PD7) |(1<<UNUSED_PD6) |(1<<TEST_PIN)    |(1<<UNUSED_PD4)   |				      (1<<TX)|        					     (1<<UNUSED_PD0);

// default to ground
	PORTB 	&= ~(1<<UNUSED_PB7)& ~(1<<UNUSED_PB6)& ~(1<<UNUSED_PB5)& ~(1<<UNUSED_PB4)& ~(1<<LCD_POWER)	  & 	 				                 ~(1<<LEBENSLICHT);
	PORTC	&= 									   ~(1<<I2C_SCL)   & ~(1<<I2C_SDA)   & ~(1<<SENSOR_SIGNAL)& ~(1<<SENSOR_POWER)&                  ~(1<<UNUSED_PC0);
	PORTD	&= ~(1<<UNUSED_PD7)& ~(1<<UNUSED_PD6)& ~(1<<TEST_PIN  )& ~(1<<UNUSED_PD4)					  						&~(1<<TX)	   & ~(1<<UNUSED_PD0);
// 
	PORTB   |=  (1 << ENC_SW);					// 	Pull up enable, switch encoder
	PORTD   |=  (1 << ENC_A) |  (1 << ENC_B);	// 	Pull up enable, encoder

// TIMER 1									// starts handler in main each 500 ms
	OCR1A	= 	TIMER_500MS	;				// timer 1, compare match
	TCCR1B	|= 	((1 << WGM12) | (1 << CS12) | (1 << CS10)); 		// clear on compare match, divide by 1024
	TIMSK1	|= 	(1 << OCIE1A);				// timer1 compare match 1 interrupt enable

//initialize encoder
   	EICRA	|=	((1<<ISC10)|(1<<ISC00)); 		// Trigger INT0 & INT1 on changing edge
  	EIMSK	|=	((1<<INT1)|(1<<INT0));   		// Enable INT0 & INT1

// init encoder switch on Port B and PIN 1, PB1
  	PCMSK0  = 	(1 << PCINT1);				// Pin change mask register, enable PCINT1 (PB1) for interrupt  
  	PCICR   = 	(1 << PCIE0);  				// PCINT0 (7:0), PB1 resp. pin 15 DIL28

// Calibrated Internal RC Oscillator
	OSCCAL  = 0x8c;							// in case, adjust for UART 

//initialize OLED Display
	PORTB |= (1<<LCD_POWER);				// turn power on LCD
	_delay_ms(10); 							// stabelize
   	lcd_init(LCD_DISP_ON);    				// init lcd and turn on
   	lcd_set_contrast(0xff);
   	lcd_clrscr();
	lcd_charMode(DOUBLESIZE);				// font size normal or large 1 or 2

#ifndef DEBUG
	welcome_lcd();							// welcome message
#endif
   	
	watchdogSetup();
	sei();									// enable all interrupts

#ifdef DEBUG								// if debugging, turn UART and LED on 
uartInitTx();							// 9600 BAUD at 8 MHz
uartPuts(">S: INIT \n");
PORTB |= (1<<LEBENSLICHT);				// turn on Lebenslicht
#endif

//	load array
	for (uint8_t tw = 1; tw <= 25; tw++) 	// load initially the data for display
		history[tw] = 150.0 - tw*8;	
	for (uint8_t tw = 26; tw <= 75; tw++) 
		history[tw] = 21.0;	
	for (uint8_t tw = 76; tw <= 100; tw++) 
		history[tw] = 21.0;	

// ################################  main loop  #####################################################
while (1) 							// endless loop
{

//################################  handler MEASURE  #####################################

		if (timer_flag && (charMode == MEASURE))	// work here if timer_flag was set (each 500 ms)
		{
												
#ifdef DEBUG 
PORTB ^= (1<<LEBENSLICHT);			// toggle Lebenslicht
#endif				
	
			if (charMode_old != charMode)		// init Measure display only once after change
			{
				charMode_old = charMode;
				lcd_clrscr();					// clear screen, takes some time
				lcd_charMode(DOUBLESIZE);		// font size small now

#ifdef DEBUG
uartPuts(">MEASURE mode\n"); 
#endif
			}

			history[0] = ((float)(getTSicTemp())/ 2047 * 200) - 50;	// TSic 306,  read temp-sensor and copy into integer array
//			history[0] = ((float)(getTSicTemp())/ 2047 * 70) - 10;	// TSic 506, read temp-sensor and copy into		
	  			
			display_temp_OLED();
	
#ifdef DEBUG
uartPuts("T= ");				// UART MELDUNG
uartPuts(display_str);			//
uartPuts(" \n");				// noch ein Linefeed und cr hinterher, uartPuts(",  \n");
#endif

			timer_flag = false;			// clear button flag
		}


//################################  handler GRAPH   ###########################################

		if (timer_flag && (charMode == GRAPH))	// work here if Logger selected
		{										

#ifdef DEBUG 
PORTB ^= (1<<LEBENSLICHT);			// toggle Lebenslicht
#endif

			if (charMode_old != charMode)		// init graph only once after change
			{
				charMode_old = charMode;
				lcd_clrscr();					// clear OLED
				lcd_charMode(NORMALSIZE);		// font size small now
				draw_coordinate_system_lcd();	// display coordinate system 

#ifdef DEBUG
uartPuts(">GRAPH mode\n");			// for debugging
#endif
				draw_graph();					// display measurement result of 100 points
												// use curser function for individual samples			
			}
			timer_flag = false;					// clear button flag
		}

//################################  handler ENCODER    #############################################

		if (((encoder_left_flag) || (encoder_right_flag))  && (charMode == GRAPH))// if encoder rotate
		{		
			display_str[0] 	= '\0';						// delete old contents
			temp_str[0]		= '\0';
			float time_ft	= 0.0;			
		
			time_ft = (float) marker * -1 / 4;
														// display min and max values at 1. line
			lcd_puts_xy(0,0,"                     ");	// clear line		
			strcat(display_str, "Marker ");				// add into string
			dtostrf(time_ft, 2, 2, temp_str);			// convert float 
			strcat(display_str, temp_str);				// add into string
			strcat(display_str, "H ");					// add space unit an space
			dtostrf(history[marker], 2, 1, temp_str);	// convert float 
			strcat(display_str, temp_str);				// add float into string
			strcat(display_str, "C");					// add unit Volts
  			lcd_puts_xy(0,0, display_str);				// now display marker string in 1. line
  			lcd_display();								// copy to display

//			voltage_pixel = history[marker];			// ???????????????????????????????????      Implementiere hier Curserfunktion    ????????
																												
//			lcd_drawLine(OFFSET_X + marker_old, OFFSET_Y, OFFSET_X + marker_old, OFFSET_Y -50, BLACK); // delete old marker indicator 
//			lcd_drawLine(OFFSET_X + marker,     OFFSET_Y, OFFSET_X + marker,     OFFSET_Y -50, WHITE); 

			marker_old = marker;						// save for later delete function

			encoder_left_flag = false;
			encoder_right_flag = false;
		}

//################################  handler POWER DOWN   ###########################################

	while (power_down_mode) 					// ISR (TIMER1_COMPA_vect), auto_sleep_counter sets this flag
	{
		PORTB &= ~(1<<LCD_POWER);				// LCD OFF
			
#ifdef DEBUG 
PORTB &= ~(1<<LEBENSLICHT);				// Lebenslicht OFF
#endif

		wdt_reset();							// reset watchdog counter
	  	set_sleep_mode(SLEEP_MODE_PWR_DOWN); 	// choose power down mode, clock for watchdog keeps runing
		sleep_mode();							// here processor terminates work.
												// but resumes work after a watchdog interrupt or pin change interrupt (button) 
						
		if (button_flag)						// wake up! if encoder button was used (power down counter)
		{										// ISR(PCINT0_vect) sets button_flag 

#ifdef DEBUG
uartPuts("BUTTON\n");				
#endif
					 
			button_flag = false;				// reset flag
			power_down_mode = false;			// terminate power down mode now, 

			PORTB |= (1<<LCD_POWER);			// turn power on LCD 3,3 V (about 7 mA)
			_delay_ms(10); 						// charge 100 nF capacitor at Vcc 3,3 V 
   			lcd_init(LCD_DISP_ON);    			// init lcd and turn on
   			lcd_set_contrast(0xff);

			charMode = GRAPH;					// resume always GRAPH mode after power down
			charMode_old = MEASURE;			
			timer_flag = true;					// and enter GRAPH handler immediatly
			break;								// now exit while 
		}
	
		if (watchdog_flag)						// handle now watchdog (each 8 seconds)
		{
			watchdog_flag = false;

#ifdef DEBUG
uartPuts("WDG 8.3s\n");			// for debugging
#endif

			if (watchdog_counter >= UPDATE)		// set update time in DEFINE, 15 min, 1 min or 16 seconds (mulitible of 8.3 seconds
			{
				PORTC |= (1<<SENSOR_POWER);			// turn sensor on
				_delay_us(60);                  	// wait TSIC
				cli();								// all interrups off 

				history[0] = ((float)(getTSicTemp())/ 2047 * 200) - 50;	// read temp-sensor TSic 306 and copy into integer array
//				history[0] = ((float)(getTSicTemp())/ 2047 * 70) - 10;	// read temp-Sensor TSic 506 and copy into array 		

				sei(); 								// interrups on
				PORTC &= ~(1<<SENSOR_POWER);		// Schalte Sensor Power aus

				for(uint8_t i = 100; i > 0 ; i--)	// shift by one element and copy [0] in to array
				{									// i = 0 present temp, i=1 recent value, i=100 old value
					history[i] = history[i-1];
				}

#ifdef DEBUG
uartPuts("WDG update\n");			// for debugging
#endif

				watchdog_counter = 0;				// clear, count up to 15 min again
			}

		}
	}  
   } // ende while
} // ende main   ##########################################  END MAIN  ##########################################

void watchdogSetup(void)					// init for watchdog timer, here used for power down and safe current
{
  	cli();
  	wdt_reset();
  	WDTCSR |= (1<<WDCE) | (1<<WDE);
  	WDTCSR  = (1<<WDIE) | (0<<WDE) | (1<<WDP3) | (1<<WDP0);  // 8.3 s / interrupt, no system reset
  	sei();
}

uint8_t calculate_min(void)					// search for minimum value in array
{
	uint8_t    c = 1,
			   location = 1;
	for (c = 1; c <= 100; c++)
    	if (history[c] < history[location])
      		location = c;
	return location;
}

uint8_t calculate_max(void)					// search for maximum value in array
{
	uint8_t    c = 1,
			   location = 1;
	for (c = 1; c <= 100; c++)
    	if (history[c] > history[location])	// here >
      		location = c;
	return location;
}


void draw_graph(void)					// read array and draw graph
{
	display_str[0] 	= '\0';				// delete old contents
	temp_str[0]		= '\0';

	float	t_max	= history[calculate_max()];		// find and store max value in array
	float	t_min	= history[calculate_min()];		// find and sotre min valuel in array
	float	y_gain;
	int		y_offset, y_pixel, y_pixel_old=0;

	y_gain =  (50 / (t_max - t_min));		// calculate gain of slope (Y = mx + b)  
											// y_gain signed integer 16 bit 							
	y_offset = (int)((-1)* y_gain * t_min);

for (uint8_t time_counter = 1; time_counter <= 100; time_counter++ ) 	// draw graph
{
	y_pixel = (int) (y_gain * history[101-time_counter] + y_offset);	// read each values multiplay with gain and add offset of slope	
							
	delta_px = 	y_pixel - y_pixel_old;
																	// draw graph of signal on display 
	lcd_drawLine(OFFSET_X + time_counter, OFFSET_Y - y_pixel, OFFSET_X + time_counter ,OFFSET_Y - y_pixel + delta_px, WHITE);
	lcd_drawLine(OFFSET_X + time_counter, OFFSET_Y - y_pixel, OFFSET_X + time_counter ,OFFSET_Y - y_pixel + delta_px +1 , WHITE); // second line thick
	y_pixel_old = y_pixel;
};
										// hier min und max 
	lcd_puts_xy(0,0,"                  ");		// clear temp line	
	strcat(display_str, "Min:");				// add unit Celsius
	dtostrf(t_min, 2, 1, temp_str);		// convert float to ascii 
	strcat(display_str, temp_str);
	strcat(display_str, "C ");
	strcat(display_str, "Max:");		// add unit Celsius		
	dtostrf(t_max, 2, 1, temp_str);		// convert float to ascii 
	strcat(display_str, temp_str);
	strcat(display_str, "C ");			// add unit Celsius
	lcd_puts_xy(0,0, display_str);

	lcd_display();						// copy buffer to LCD
}

void welcome_lcd(void)
{									// symbol Tippel-Quad-Symbol
	for(uint8_t k=10; k<20; k++)	// enlarge symbol during start up
	{
	lcd_clrscr();
 	lcd_drawLine(CENTRE_X-30, CENTRE_Y, CENTRE_X+30, CENTRE_Y, WHITE);	// x axis
 	lcd_drawLine(CENTRE_X, CENTRE_Y-30, CENTRE_X, CENTRE_Y+30, WHITE);	// y axis
																		// 20 m quad
	lcd_drawLine(CENTRE_X, CENTRE_Y-27*k/20, CENTRE_X+27*k/20, CENTRE_Y, WHITE);	// top right 
	lcd_drawLine(CENTRE_X, CENTRE_Y-27*k/20, CENTRE_X-27*k/20, CENTRE_Y, WHITE);	// top left
	lcd_drawLine(CENTRE_X+27*k/20, CENTRE_Y, CENTRE_X, CENTRE_Y+27*k/20, WHITE);	// bottom right
	lcd_drawLine(CENTRE_X-27*k/20, CENTRE_Y, CENTRE_X, CENTRE_Y+27*k/20, WHITE);	// bottom left
																		// 15 m quad
	lcd_drawLine(CENTRE_X, CENTRE_Y-20*k/20, CENTRE_X+20*k/20, CENTRE_Y, WHITE);	// top right 
	lcd_drawLine(CENTRE_X, CENTRE_Y-20*k/20, CENTRE_X-20*k/20, CENTRE_Y, WHITE);	// top left
	lcd_drawLine(CENTRE_X+20*k/20, CENTRE_Y, CENTRE_X, CENTRE_Y+20*k/20, WHITE);	// bottom right
	lcd_drawLine(CENTRE_X-20*k/20, CENTRE_Y, CENTRE_X, CENTRE_Y+20*k/20, WHITE);	// bottom left
																		// 10 m quad
	lcd_drawLine(CENTRE_X, CENTRE_Y-15*k/20, CENTRE_X+15*k/20, CENTRE_Y, WHITE);	// top right 
	lcd_drawLine(CENTRE_X, CENTRE_Y-15*k/20, CENTRE_X-15*k/20, CENTRE_Y, WHITE);	// top left
	lcd_drawLine(CENTRE_X+15*k/20, CENTRE_Y, CENTRE_X, CENTRE_Y+15*k/20, WHITE);	// bottom right
	lcd_drawLine(CENTRE_X-15*k/20, CENTRE_Y, CENTRE_X, CENTRE_Y+15*k/20, WHITE);	// bottom left

	lcd_display();			
	_delay_ms(10);
	}

	_delay_ms(2000);
	lcd_clrscr();

	lcd_charMode(NORMALSIZE);		// select character size
	lcd_puts_xy(7,3,"Welcome");  	// welcome message to begin with
	lcd_puts_xy(2,5,"Clemens Verstappen");  

	lcd_display();
	_delay_ms(1500);
	lcd_clrscr();
}

void draw_coordinate_system_lcd(void)		// draw coordinate system 
{
 // Y scale
//	lcd_puts_xy(0,1,"15");	// Y-scale marking 15 volts
//	lcd_puts_xy(0,3,"10");
//	lcd_puts_xy(0,5," 5");
	lcd_puts_xy(0,7," 0");

	for(uint8_t k=1; k<4; k++)				// makes 30 pixel / 10 V
	lcd_drawLine(OFFSET_X-4,OFFSET_Y - 15*k,OFFSET_X,OFFSET_Y - 15*k,WHITE);	// skalierung
	lcd_drawLine(OFFSET_X,7,OFFSET_X,62,WHITE);	 			// y axis

// X scale
//	for(uint8_t k=1; k<3; k++)				// makes 100 pixel / 2s  = 50 px/s
//	lcd_drawLine(OFFSET_X + 50*k, OFFSET_Y, OFFSET_X+50*k, OFFSET_Y + 3,WHITE);	// skalierung 1 Sekunde

// X scale
//	for(uint8_t k=1; k<5; k++)				// makes 100 pixel / 2s  = 50px/s
//	lcd_drawLine(OFFSET_X + 25*k, OFFSET_Y, OFFSET_X+25*k, OFFSET_Y + 1,WHITE);	// skalierung 500 mS

// 	lcd_drawLine(OFFSET_X-5,OFFSET_Y,123,OFFSET_Y,WHITE);	// X Achse/time 

  	lcd_display();		// cpy buffer to LC display via I2C
}

uint16_t getTSicTemp(void)
{
	uint8_t 	i, j ,parity, strobe_time, strobe_count;
	uint16_t	temperatur=0;

	PORTC |= (1<<SENSOR_POWER);			// turn sensor on
	_delay_us(60);                  	// wait TSIC
	cli();								// all interrups off 

    while(PINC & (1<<SENSOR_SIGNAL));           // warten auf high
    for (i=0;i<2;i++)                           // 2 Bytes Daten
    {
        strobe_time = 0;                        // fr Messung lschen
        while(PINC & (1<<SENSOR_SIGNAL));       // warten auf low
        										// Startbit, ab hier messen
        while(!( PINC & (1<<SENSOR_SIGNAL)))    // Low - Zeit messen
        {
            strobe_time++;                      // messen bitlnge
            if (strobe_time==0) return 999;     // berlauf, error
        }                                       // strobe_time hat ca. Wert 62 = 62s bei 40MHz
        parity = 0;
        for (j=0;j<8;j++)                       // 8 Datenbits
        {
            temperatur <<= 1;                   // nach links verschieben,
                                                // Bit 0 wird automatisch auf 0 gesetzt
            strobe_count = strobe_time;         // Zhler laden
            while(PINC & (1<<SENSOR_SIGNAL));   // auf fallende Flanke warten

            while(strobe_count--) { Nop(); }    // hier die gemessene Bitzeit warten

            if (PINC & (1<<SENSOR_SIGNAL))      // PIN messen und Bit 0 setzen wenn high
            {
                temperatur |= 0x01;
                parity++;                       // Paritt gleich mitzhlen
            }
            while(!( PINC & (1<<SENSOR_SIGNAL)));                  // weiter wenn high
        } 										// ende bitcount
        										// Parity Bit
        strobe_count = strobe_time;             // Zhler laden
        while(PINC & (1<<SENSOR_SIGNAL));       // auf Low warten

        while(strobe_count--) { Nop(); }        // hier die gemessene Bitzeit warten

        if (PINC & (1<<SENSOR_SIGNAL)) parity++;// Parity Bit dazurechnen

        if (parity % 2 != 0) return 999;        // Parity Fehler (muss gerade Zahl sein)

        while(!( PINC & (1<<SENSOR_SIGNAL)));   // weiter wenn high

    } 											// ende Bytecount

	sei(); 								// interrups on
	PORTC &= ~(1<<SENSOR_POWER);		// Schalte Sensor Power aus
    return temperatur;					// Umrechnung in Temperatur, gleich in Festkomma mit einer Nachkommastelle
}

void display_temp_OLED(void)
{
	display_str[0] 	= '\0';				// delete old contents
	temp_str[0]		= '\0';
	lcd_puts_xy(0,1,"Temperatur");
	lcd_puts_xy(0,4,"          ");		// clear temp line	
		
	dtostrf(history[0], 2, 1, temp_str);	// convert float to ascii - TSic306
//	dtostrf(history[0], 2, 2, temp_str);	// convert float to ascii - TSic506 sensor two digits  
	strcat(display_str, temp_str);		// add float into string
	strcat(display_str, " C");			// add unit Celsius
	lcd_puts_xy(4,4, display_str);		// now display marker string 
	lcd_display();						// copy to display
}

ISR(WDT_vect)							// Interrupt routine watchdog tripps 128 kHz clocksignal
{
	watchdog_flag = true;
	watchdog_counter++;					// each count equals 8.3 seconds
}			

SIGNAL(TIMER1_COMPA_vect) 				// MAIN INTERVAL, start for main loop 500 ms
{										 
	timer_flag = true;
	auto_sleep_counter++;
	if (auto_sleep_counter >= AUTO_OFF)	// swap into sleep mode and turn off display
	{
	    power_down_mode = true;			// switch off LCD and Controler in to Power down mode
//		auto_sleep_counter = 60;		// avoid overflow
		button_flag = false;			// in case if it was set
	}
} 

ISR(PCINT0_vect)  						// encoder push button
{
	if	(!(PINB & (1<<PINB1)))			// check if low now (not high), I know ...
	{
		charMode++;
		if (charMode == 2)				// only MEAURE and GRAPH
			charMode = 0;

		button_flag = true;
	}
	auto_sleep_counter = 0;				// reset here, avoid auto switch off
}

ISR(INT0_vect)    						// rotary encoder A
{
	if (((PIND) & (1<<PIND3)) != A_set)	// check level of other encoder input
    A_set = !A_set;						// has it changed,  - store result

 	if (A_set && !B_set)				// level of A and B are different at slope of A
	{
		marker++;						// action: update marker. Pixel on x-axis (1 to 100 points )
		if (marker >= 100) 
			marker = 100;	

		encoder_right_flag = true;		// flag for main program
	}
	auto_sleep_counter = 0;				// reset here
}

ISR(INT1_vect)    						// rotary encoder B
{
	if (((PIND) & (1<<PIND2)) != B_set)	// check level of other encoder input
    B_set = !B_set;						// has it changed,  - store result

	if (B_set && !A_set)				// level of A and B are different at slope of B
	{
		marker--;						// action: update marker. Pixel on x-axis (1 to 100 points )
		if (marker <= 1) 
			marker = 1;

		encoder_left_flag = true;		// flag for main program
	}
	auto_sleep_counter = 0;
}

